import asyncio

from onvif import ONVIFCamera


async def media_profile_configuration():
    """
    A media profile consists of configuration entities such as video/audio
    source configuration, video/audio encoder configuration,
    or PTZ configuration. This use case describes how to change one
    configuration entity which has been already added to the media profile.
    """

    # Create the media service
    mycam = ONVIFCamera('192.168.0.112', 80, 'admin', '12345')
    await mycam.update_xaddrs()
    media_service = mycam.create_media_service()

    profiles = await media_service.GetProfiles()

    # Use the first profile and Profiles have at least one
    token = profiles[0].token

    # Get all video encoder configurations
    configurations_list = await media_service.GetVideoEncoderConfigurations()

    # Use the first profile and Profiles have at least one
    video_encoder_configuration = configurations_list[0]

    # Get video encoder configuration options
    options = await media_service.GetVideoEncoderConfigurationOptions({'ProfileToken': token})

    # Setup stream configuration
    video_encoder_configuration.Encoding = 'H264'
    # Setup Resolution
    video_encoder_configuration.Resolution.Width = (
        options.H264.ResolutionsAvailable[0].Width)
    video_encoder_configuration.Resolution.Height = (
        options.H264.ResolutionsAvailable[0].Height)
    # Setup Quality
    video_encoder_configuration.Quality = options.QualityRange.Min
    # Setup FramRate
    video_encoder_configuration.RateControl.FrameRateLimit = (
        options.H264.FrameRateRange.Min)
    # Setup EncodingInterval
    video_encoder_configuration.RateControl.EncodingInterval = (
        options.H264.EncodingIntervalRange.Min)
    # Setup Bitrate
    video_encoder_configuration.RateControl.BitrateLimit = (
        options.Extension.H264[0].BitrateRange[0].Min[0])

    # Create request type instance
    request = media_service.create_type('SetVideoEncoderConfiguration')
    request.Configuration = video_encoder_configuration
    # ForcePersistence is obsolete and should always be assumed to be True
    request.ForcePersistence = True

    # Set the video encoder configuration
    # await media_service.SetVideoEncoderConfiguration(request)

if __name__ == '__main__':
    loop = asyncio.get_event_loop()
    loop.run_until_complete(media_profile_configuration())



import datetime
import logging
import os.path

import zeep.client
import zeep.exceptions
import zeep.helpers
import zeep.proxy
import zeep.wsse.username

from . import exceptions
from . import wsdl


logger = logging.getLogger('onvif')
logging.basicConfig(level=logging.INFO)
logging.getLogger('zeep.client').setLevel(logging.CRITICAL)


def safe_func(func):
    """Ensure methods to raise an ONVIFError when something was wrong."""

    def wrapped(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            raise exceptions.ONVIFError(e)

    return wrapped


class UsernameDigestTokenDtDiff(zeep.wsse.username.UsernameToken):
    """UsernameDigestToken class, with a time offset parameter that can be adjusted.

    This allows authentication on cameras without being time synchronized.
    Please note that using NTP on both end is the recommended solution,
    this should only be used in "safe" environments.
    """

    def __init__(self, user, passw, dt_diff=None, **kwargs):
        super().__init__(user, passw, **kwargs)
        # Date/time difference in datetime.timedelta
        self.dt_diff = dt_diff

    def apply(self, envelope, headers):
        old_created = self.created
        if self.created is None:
            self.created = datetime.datetime.utcnow()
        if self.dt_diff is not None:
            self.created += self.dt_diff
        result = super().apply(envelope, headers)
        self.created = old_created
        return result


class ONVIFService:
    """Python Implemention for ONVIF Service.

    Services List:
        Analytics
        DeviceIO
        DeviceMgmt
        Events
        Imaging
        Media
        Notification
        PTZ
        PullPointSubscription
        Receiver
        Recording
        Replay
        Search
        Subscription

    There are two ways to pass parameter to services methods
    1. Dict
        params = {'Name': 'NewHostName'}
        device_service.SetHostname(params)
    2. Type Instance
        params = device_service.create_type('SetHostname')
        params.Hostname = 'NewHostName'
        device_service.SetHostname(params)

    >>> from onvif import ONVIFService
    >>> device_service = ONVIFService('http://192.168.0.112/onvif/device_service',
    ...                           'admin', 'foscam',
    ...                           '/etc/onvif/wsdl/devicemgmt.wsdl')
    >>> ret = device_service.GetHostname()
    >>> print ret.FromDHCP
    >>> print ret.Name
    >>> device_service.SetHostname(dict(Name='newhostname'))
    >>> ret = device_service.GetSystemDateAndTime()
    >>> print ret.DaylightSavings
    >>> print ret.TimeZone
    >>> dict_ret = device_service.to_dict(ret)
    >>> print dict_ret['TimeZone']
    """

    @safe_func
    def __init__(
        self,
        xaddr,
        user,
        passwd,
        url,
        binding_name,
        encrypt=True,
        dt_diff=None,
    ):
        if not os.path.isfile(url):
            raise exceptions.ONVIFError(f"{url!r} doesn't exist!")

        self.url = url
        self.xaddr = xaddr

        # Create security token
        wsse = UsernameDigestTokenDtDiff(
            user, passwd, dt_diff=dt_diff, use_digest=encrypt,
        )

        # Client settings
        settings = zeep.client.Settings()
        settings.strict = False
        settings.xml_huge_tree = True

        # Create client
        self._client = zeep.client.AsyncClient(
            wsdl=url,
            wsse=wsse,
            settings=settings,
        )

        # Create service proxy, it's a workaround as zeep still
        # doesn't support AsyncServiceProxy for AsyncClient
        binding = self._client.wsdl.bindings.get(binding_name)
        if binding is None:
            raise exceptions.ONVIFError(f'Binding was not found: {binding_name!r}')

        self._service_proxy = zeep.proxy.AsyncServiceProxy(
            self._client,
            binding,
            address=self.xaddr,
        )

        # Set soap header for authentication
        self.user = user
        self.passwd = passwd
        # Indicate wether password digest is needed
        self.encrypt = encrypt
        self.dt_diff = dt_diff

        namespace = binding_name[binding_name.find('{') + 1: binding_name.find('}')]
        available_ns = self._client.namespaces
        active_ns = (
            list(available_ns.keys())[list(available_ns.values()).index(namespace)]
            or 'ns0'
        )
        self.create_type = lambda x: self._client.get_element(active_ns + ':' + x)()

    async def close(self):
        """Close the transport session."""
        if self._client:
            await self._client.transport.aclose()

    @staticmethod
    @safe_func
    def to_dict(zeepobject):
        """Convert a WSDL Type instance into a dictionary."""
        return {} if zeepobject is None else zeep.helpers.serialize_object(zeepobject)

    def __getattr__(self, name):
        """Call the real ONVIF Service operations.

        See the official wsdl definition for the
        APIs detail(API name, request parameters,
        response parameters, parameter types, etc...)
        """

        def service_wrapper(func):
            """Wrap service call."""

            @safe_func
            def wrapped(params=None):
                def call(params=None):
                    if params is None:
                        params = {}
                    else:
                        params = ONVIFService.to_dict(params)
                    try:
                        ret = func(**params)
                    except TypeError:
                        ret = func(params)
                    return ret

                return call(params)

            return wrapped

        builtin = name.startswith('__') and name.endswith('__')
        if builtin:
            return self.__dict__[name]

        return service_wrapper(getattr(self._service_proxy, name))


class ONVIFCamera:
    """Python Implemention ONVIF compliant device.

    This class integrates onvif services.

    adjust_time parameter allows authentication on cameras without being time synchronized.
    Please note that using NTP on both end is the recommended solution,
    this should only be used in "safe" environments.
    Also, this cannot be used on AXIS camera, as every request is authenticated,
    contrary to ONVIF standard.

    >>> from onvif import ONVIFCamera
    >>> mycam = ONVIFCamera('192.168.0.112', 80, 'admin', '12345')
    >>> mycam.devicemgmt.GetServices(False)
    >>> media_service = mycam.create_media_service()
    >>> ptz_service = mycam.create_ptz_service()
    # Get PTZ Configuration:
    >>> mycam.ptz.GetConfiguration()
    # Another way:
    >>> ptz_service.GetConfiguration()
    """
    def __init__(
        self,
        host,
        port,
        user,
        passwd,
        wsdl_dir=None,
        encrypt=True,
        adjust_time=False,
    ):
        # TODO: Handle case-sensivity
        os.environ.pop('http_proxy', None)
        os.environ.pop('https_proxy', None)
        self.host = host
        self.port = int(port)
        self.user = user
        self.passwd = passwd
        self.wsdl_dir = wsdl_dir
        self.encrypt = encrypt
        self.adjust_time = adjust_time
        self.dt_diff = None
        self.xaddrs = {}

        # Active service client container
        self.services = {}

        self.to_dict = ONVIFService.to_dict

        # TODO: Better to handle with pathlib
        if self.wsdl_dir is None:
            self.wsdl_dir = os.path.join(os.path.dirname(__file__), 'wsdl')

    async def update_xaddrs(self):
        """Update xaddrs for services."""
        self.dt_diff = None
        devicemgmt = self.create_devicemgmt_service()
        if self.adjust_time:
            sys_date = await devicemgmt.GetSystemDateAndTime()
            cdate = sys_date.UTCDateTime
            cam_date = datetime.datetime(
                cdate.Date.Year,
                cdate.Date.Month,
                cdate.Date.Day,
                cdate.Time.Hour,
                cdate.Time.Minute,
                cdate.Time.Second,
            )
            self.dt_diff = cam_date - datetime.datetime.utcnow()

        # Get XAddr of services on the device
        self.xaddrs = {}
        capabilities = await devicemgmt.GetCapabilities({'Category': 'All'})
        for name in capabilities:
            capability = capabilities[name]
            try:
                if name.lower() in wsdl.SERVICES and capability is not None:
                    namespace = wsdl.SERVICES[name.lower()]['ns']
                    self.xaddrs[namespace] = capability['XAddr']
            except Exception:
                logger.exception('Unexpected service type')

    async def create_pullpoint_subscription(self):
        """Create a pullpoint subscription."""
        try:
            events = self.create_events_service()
            pullpoint = await events.CreatePullPointSubscription()

            self.xaddrs[
                'http://www.onvif.org/ver10/events/wsdl/PullPointSubscription',
            ] = pullpoint.SubscriptionReference.Address._value_1
        except zeep.exceptions.Fault:
            return False
        return True

    async def close(self):
        """Close all transports."""

        for service in self.services.values():
            await service.close()

    def get_definition(self, name, port_type=None):
        """Return xaddr and wsdl of specified service."""

        # Check if the service is supported
        if name not in wsdl.SERVICES:
            raise exceptions.ONVIFError(f'Unknown service {name!r}')

        wsdl_file = wsdl.SERVICES[name]['wsdl']
        namespace = wsdl.SERVICES[name]['ns']
        binding = wsdl.SERVICES[name]['binding']

        binding_name = f'{{{namespace}}}{binding}'

        if port_type:
            namespace += '/' + port_type

        # TODO: Cache or load asynchronously
        wsdlpath = os.path.join(self.wsdl_dir, wsdl_file)
        if not os.path.isfile(wsdlpath):
            raise exceptions.ONVIFError(f'No such file: {wsdlpath!r}')

        # XAddr for devicemgmt is fixed
        if name == 'devicemgmt':
            host = self.host
            # If scheme is missing, then treat the scheme as http://
            if not self.host.startswith('http://') or not self.host.startswith('https://'):
                host = f'http://{self.host}'

            xaddr = f'{host}:{self.port}/onvif/device_service'
            return xaddr, wsdlpath, binding_name

        # Get other XAddr
        xaddr = self.xaddrs.get(namespace)
        if not xaddr:
            raise exceptions.ONVIFError(f"Device doesn't support service: {name!r}")

        return xaddr, wsdlpath, binding_name

    def create_onvif_service(self, name, port_type=None) -> ONVIFService:
        """Create ONVIF service and update service registry."""

        name = name.lower()
        xaddr, wsdl_file, binding_name = self.get_definition(name, port_type)

        # Don't re-create bindings if the xaddr remains the same,
        # the xaddr can change when a new PullPointSubscription is created
        binding = self.services.get(binding_name)
        if binding and binding.xaddr == xaddr:
            return binding

        service = ONVIFService(
            xaddr,
            self.user,
            self.passwd,
            wsdl_file,
            binding_name,
            encrypt=self.encrypt,
            dt_diff=self.dt_diff,
        )

        self.services[binding_name] = service

        return service

    def create_analytics_service(self) -> ONVIFService:
        """Create analytics service."""
        return self.create_onvif_service('analytics')

    def create_deviceio_service(self) -> ONVIFService:
        """Create deviceio service."""
        return self.create_onvif_service('deviceio')

    def create_devicemgmt_service(self) -> ONVIFService:
        """Create devicemgmt service."""
        return self.create_onvif_service('devicemgmt')

    def create_events_service(self) -> ONVIFService:
        """Create events service."""
        return self.create_onvif_service('events')

    def create_imaging_service(self) -> ONVIFService:
        """Create imaging service."""
        return self.create_onvif_service('imaging')

    def create_media_service(self) -> ONVIFService:
        """Create meida service."""
        return self.create_onvif_service('media')

    def create_notification_service(self) -> ONVIFService:
        """Create notification service."""
        return self.create_onvif_service('notification')

    def create_ptz_service(self) -> ONVIFService:
        """Create ptz service."""
        return self.create_onvif_service('ptz')

    def create_pullpoint_service(self) -> ONVIFService:
        """Create pullpoint service."""
        return self.create_onvif_service(
            'pullpoint',
            port_type='PullPointSubscription',
        )

    def create_receiver_service(self) -> ONVIFService:
        """Create receiver service."""
        return self.create_onvif_service('receiver')

    def create_recording_service(self) -> ONVIFService:
        """Create recording service."""
        return self.create_onvif_service('recording')

    def create_replay_service(self) -> ONVIFService:
        """Create replay service."""
        return self.create_onvif_service('replay')

    def create_search_service(self) -> ONVIFService:
        """Create search service."""
        return self.create_onvif_service('search')

    def create_subscription_service(self, port_type=None) -> ONVIFService:
        """Create subscription service."""
        return self.create_onvif_service(
            'subscription',
            port_type=port_type,
        )



#!/usr/bin/python

import argparse
import ast
import cmd
import os.path
import re

import zeep.exceptions
import zeep.xsd

from . import client
from . import exceptions
from . import wsdl


SUPPORTED_SERVICES = wsdl.SERVICES.keys()


class ThrowingArgumentParser(argparse.ArgumentParser):
    """Exception throwing argument parser."""

    def error(self, message):
        usage = self.format_usage()
        raise ValueError(f'{message}\n{usage}')


def success(message):
    """Print success message."""
    print(f'True: {message!r}')


def error(message):
    """Print error message."""
    print(f'False: {message!r}')


class ONVIFCLI(cmd.Cmd):
    """ONVIF CLI class."""

    prompt = 'ONVIF >>> '
    client = None
    cmd_parser = None

    def setup(self, args):
        """Setup parser."""
        # Create onvif camera client
        self.client = client.ONVIFCamera(
            args.host,
            args.port,
            args.user,
            args.password,
            args.wsdl,
            encrypt=args.encrypt,
        )

        # Create cmd argument parser
        self.create_cmd_parser()

    def create_cmd_parser(self):
        """Create parser to parse CMD, params is optional."""
        cmd_parser = ThrowingArgumentParser(
            prog='ONVIF CMD',
            usage='CMD service operation [params]',
        )
        cmd_parser.add_argument('service')
        cmd_parser.add_argument('operation')
        cmd_parser.add_argument(
            'params',
            default='{}',
            nargs=argparse.REMAINDER,
        )
        self.cmd_parser = cmd_parser

    def do_cmd(self, line):
        """Usage: CMD service operation [parameters]."""
        try:
            args = self.cmd_parser.parse_args(line.split())
        except ValueError as err:
            return error(err)

        # Check if args.service is valid
        if args.service not in SUPPORTED_SERVICES:
            return error(f'No Service: {args.service}')

        args.params = ''.join(args.params)
        # params is optional
        if not args.params.strip():
            args.params = '{}'

        # params must be a dictionary format string
        match = re.match(r'^.*?(\{.*\}).*$', args.params)
        if not match:
            return error('Invalid params')

        try:
            args.params = dict(ast.literal_eval(match.group(1)))
        except ValueError as e:
            return error(f'Invalid params: {e!r}')

        try:
            # Get ONVIF service
            service = self.client.get_service(args.service)
            # Actually execute the command and get the response
            response = getattr(service, args.operation)(args.params)
        except zeep.exceptions.LookupError:
            return error(f'No Operation: {args.operation}')
        except Exception as err:
            return error(err)

        if isinstance(response, (zeep.xsd.String, bool)):
            return success(response)
        # Try to convert instance to dictionary
        try:
            success(client.ONVIFService.to_dict(response))
        except exceptions.ONVIFError:
            error({})

    def complete_cmd(self, text, line, begidx, endidx):
        """Complete command."""
        if not text:
            completions = SUPPORTED_SERVICES[:]
        else:
            completions = [
                key
                for key in SUPPORTED_SERVICES
                if key.startswith(text)
            ]
        return completions

    def emptyline(self):
        """Empty line."""
        return ''

    def do_EOF(self, line):  # noqa: N802
        """End of file."""
        return True


def create_parser():
    """Create parser."""
    parser = ThrowingArgumentParser(description=__doc__)
    # Dealwith dependency for service, operation and params
    parser.add_argument(
        'service',
        nargs='?',
        help='Service defined by ONVIF WSDL document',
    )
    parser.add_argument(
        'operation',
        nargs='?',
        default='',
        help='Operation to be execute defined by ONVIF WSDL document',
    )
    parser.add_argument(
        'params',
        default='',
        nargs='?',
        help='JSON format params passed to the operation. E.g., "{"Name": "NewHostName"}"',
    )
    parser.add_argument(
        '--host',
        required=True,
        help='ONVIF camera host, e.g. 192.168.2.123, www.example.com',
    )
    parser.add_argument(
        '--port',
        default=80,
        type=int,
        help='Port number for camera, default: 80',
    )
    parser.add_argument(
        '-u',
        '--user',
        required=True,
        help='Username for authentication',
    )
    parser.add_argument(
        '-a',
        '--password',
        required=True,
        help='Password for authentication',
    )
    parser.add_argument(
        '-w',
        '--wsdl',
        default=os.path.join(os.path.dirname(os.path.dirname(__file__)), 'wsdl'),
        help='directory to store ONVIF WSDL documents',
    )
    parser.add_argument(
        '-e',
        '--encrypt',
        default='False',
        help='Encrypt password or not',
    )
    parser.add_argument(
        '-v',
        '--verbose',
        action='store_true',
        help='increase output verbosity',
    )
    parser.add_argument(
        '--cache-location',
        dest='cache_location',
        default='/tmp/onvif/',
        help='location to cache suds objects, default to /tmp/onvif/',
    )
    parser.add_argument(
        '--cache-duration',
        dest='cache_duration',
        help='how long will the cache be exist',
    )

    return parser


def main():
    """Main entrypoint."""
    # Create argument parser
    parser = create_parser()
    try:
        args = parser.parse_args()
    except ValueError as err:
        print(str(err))
        return
    # Also need parse configuration file.

    # Interactive command loop
    cli = ONVIFCLI(stdin=input)
    cli.setup(args)
    if args.service:
        cmd = ' '.join(['cmd', args.service, args.operation, args.params])
        cli.onecmd(cmd)
    # Execute command specified and exit
    else:
        cli.cmdloop()


if __name__ == '__main__':
    main()



#!/usr/bin/python

import unittest

import onvif


CAM_HOST = '172.20.9.84'
CAM_PORT = 80
CAM_USER = 'root'
CAM_PASS = 'password'

DEBUG = False


def log(ret):
    if DEBUG:
        print(ret)


class TestDevice(unittest.TestCase):

    # Class level cam. Run this test more efficiently..
    cam = onvif.ONVIFCamera(CAM_HOST, CAM_PORT, CAM_USER, CAM_PASS)

    # ***************** Test Capabilities ***************************
    def test_wsd_url(self):
        self.cam.devicemgmt.GetWsdlUrl()

    def test_get_services(self):
        """Returns a cllection of the devices services.

        Aand possibly their available capabilities.
        """
        params = {'IncludeCapability': True}
        self.cam.devicemgmt.GetServices(params)
        params = self.cam.devicemgmt.create_type('GetServices')
        params.IncludeCapability = False
        self.cam.devicemgmt.GetServices(params)

    def test_get_service_capabilities(self):
        """Returns the capabilities of the devce service."""
        ret = self.cam.devicemgmt.GetServiceCapabilities()
        ret.Network.IPFilter

    def test_get_capabilities(self):
        """Probides a backward compatible interface for the base capabilities."""
        categorys = ['PTZ', 'Media', 'Imaging',
                     'Device', 'Analytics', 'Events']
        self.cam.devicemgmt.GetCapabilities()
        for category in categorys:
            self.cam.devicemgmt.GetCapabilities({'Category': category})

        with self.assertRaises(onvif.ONVIFError):
            self.cam.devicemgmt.GetCapabilities({'Category': 'unknown'})

    def test_get_hostname(self):
        """Get the hostname from a device."""
        self.cam.devicemgmt.GetHostname()

    def test_set_hostname(self):
        """Set the hostname on a device.

        A device shall accept strings formated according to
        RFC 1123 section 2.1 or alternatively to RFC 952,
        other string shall be considered as invalid strings
        """
        pre_host_name = self.cam.devicemgmt.GetHostname()

        self.cam.devicemgmt.SetHostname({'Name': 'testHostName'})
        self.assertEqual(self.cam.devicemgmt.GetHostname().Name, 'testHostName')

        self.cam.devicemgmt.SetHostname({'Name': pre_host_name.Name})

    def test_set_hostname_from_dhcp(self):
        """Controls whether the hostname shall be retrieved from DHCP."""
        ret = self.cam.devicemgmt.SetHostnameFromDHCP({'FromDHCP': False})
        self.assertTrue(isinstance(ret, bool))

    def test_get_dns(self):
        """Gets the DNS setting from a device."""
        ret = self.cam.devicemgmt.GetDNS()
        self.assertTrue(hasattr(ret, 'FromDHCP'))
        if not ret.FromDHCP and len(ret.DNSManual) > 0:
            log(ret.DNSManual[0].Type)
            log(ret.DNSManual[0].IPv4Address)

    def test_set_dns(self):
        """Set the DNS settings on a device."""
        self.cam.devicemgmt.SetDNS({'FromDHCP': False})

    def test_get_ntp(self):
        """Get the NTP settings from a device."""
        ret = self.cam.devicemgmt.GetNTP()
        if ret.FromDHCP:
            self.assertTrue(hasattr(ret, 'NTPManual'))
            log(ret.NTPManual)

    def test_set_ntp(self):
        """Set the NTP setting."""
        self.cam.devicemgmt.SetNTP({'FromDHCP': False})

    def test_get_dynamic_dns(self):
        """Get the dynamic DNS setting."""
        ret = self.cam.devicemgmt.GetDynamicDNS()
        log(ret)

    def test_set_dynamic_dns(self):
        """Set the dynamic DNS settings on a device."""
        self.cam.devicemgmt.GetDynamicDNS()
        self.cam.devicemgmt.SetDynamicDNS({'Type': 'NoUpdate', 'Name': None, 'TTL': None})


if __name__ == '__main__':
    unittest.main()



SERVICES = {
    'devicemgmt': {
        'ns': 'http://www.onvif.org/ver10/device/wsdl',
        'wsdl': 'devicemgmt.wsdl',
        'binding': 'DeviceBinding',
    },
    'media': {
        'ns': 'http://www.onvif.org/ver10/media/wsdl',
        'wsdl': 'media.wsdl',
        'binding': 'MediaBinding',
    },
    'ptz': {
        'ns': 'http://www.onvif.org/ver20/ptz/wsdl',
        'wsdl': 'ptz.wsdl',
        'binding': 'PTZBinding',
    },
    'imaging': {
        'ns': 'http://www.onvif.org/ver20/imaging/wsdl',
        'wsdl': 'imaging.wsdl',
        'binding': 'ImagingBinding',
    },
    'deviceio': {
        'ns': 'http://www.onvif.org/ver10/deviceIO/wsdl',
        'wsdl': 'deviceio.wsdl',
        'binding': 'DeviceIOBinding',
    },
    'events': {
        'ns': 'http://www.onvif.org/ver10/events/wsdl',
        'wsdl': 'events.wsdl',
        'binding': 'EventBinding',
    },
    'pullpoint': {
        'ns': 'http://www.onvif.org/ver10/events/wsdl',
        'wsdl': 'events.wsdl',
        'binding': 'PullPointSubscriptionBinding',
    },
    'notification': {
        'ns': 'http://www.onvif.org/ver10/events/wsdl',
        'wsdl': 'events.wsdl',
        'binding': 'NotificationProducerBinding',
    },
    'subscription': {
        'ns': 'http://www.onvif.org/ver10/events/wsdl',
        'wsdl': 'events.wsdl',
        'binding': 'SubscriptionManagerBinding',
    },
    'analytics': {
        'ns': 'http://www.onvif.org/ver20/analytics/wsdl',
        'wsdl': 'analytics.wsdl',
        'binding': 'AnalyticsEngineBinding',
    },
    'recording': {
        'ns': 'http://www.onvif.org/ver10/recording/wsdl',
        'wsdl': 'recording.wsdl',
        'binding': 'RecordingBinding',
    },
    'search': {
        'ns': 'http://www.onvif.org/ver10/search/wsdl',
        'wsdl': 'search.wsdl',
        'binding': 'SearchBinding',
    },
    'replay': {
        'ns': 'http://www.onvif.org/ver10/replay/wsdl',
        'wsdl': 'replay.wsdl',
        'binding': 'ReplayBinding',
    },
    'receiver': {
        'ns': 'http://www.onvif.org/ver10/receiver/wsdl',
        'wsdl': 'receiver.wsdl',
        'binding': 'ReceiverBinding',
    },
}



import pytest

import onvif
import onvif.client


def test_safe_func():
    @onvif.client.safe_func
    def maybe_raise(r=False):
        if r:
            raise Exception('oops')

        return 'ok'

    assert maybe_raise() == 'ok'

    with pytest.raises(
        onvif.ONVIFError,
        match='oops',
    ):
        maybe_raise(True)



import asyncio
import sys

import onvif


IP = '192.168.0.100'  # Camera IP address
PORT = 10080  # Port
USER = 'admin'  # Username
PASS = 'password'  # Password

XMAX = 1
XMIN = -1
YMAX = 1
YMIN = -1
moverequest = None
ptz = None
active = False


async def do_move(ptz, request):
    # Start continuous move
    global active
    if active:
        await ptz.Stop({'ProfileToken': request.ProfileToken})
    active = True
    await ptz.ContinuousMove(request)


async def move_up(ptz, request):
    print('move up...')
    request.Velocity.PanTilt.x = 0
    request.Velocity.PanTilt.y = YMAX
    await do_move(ptz, request)


async def move_down(ptz, request):
    print('move down...')
    request.Velocity.PanTilt.x = 0
    request.Velocity.PanTilt.y = YMIN
    await do_move(ptz, request)


async def move_right(ptz, request):
    print('move right...')
    request.Velocity.PanTilt.x = XMAX
    request.Velocity.PanTilt.y = 0
    await do_move(ptz, request)


async def move_left(ptz, request):
    print('move left...')
    request.Velocity.PanTilt.x = XMIN
    request.Velocity.PanTilt.y = 0
    await do_move(ptz, request)


async def move_upleft(ptz, request):
    print('move up left...')
    request.Velocity.PanTilt.x = XMIN
    request.Velocity.PanTilt.y = YMAX
    await do_move(ptz, request)


async def move_upright(ptz, request):
    print('move up left...')
    request.Velocity.PanTilt.x = XMAX
    request.Velocity.PanTilt.y = YMAX
    await do_move(ptz, request)


async def move_downleft(ptz, request):
    print('move down left...')
    request.Velocity.PanTilt.x = XMIN
    request.Velocity.PanTilt.y = YMIN
    await do_move(ptz, request)


async def move_downright(ptz, request):
    print('move down left...')
    request.Velocity.PanTilt.x = XMAX
    request.Velocity.PanTilt.y = YMIN
    await do_move(ptz, request)


async def setup_move():
    mycam = onvif.ONVIFCamera(IP, PORT, USER, PASS)
    await mycam.update_xaddrs()
    # Create media service object
    media = mycam.create_media_service()

    # Create ptz service object
    global ptz
    ptz = mycam.create_ptz_service()

    # Get target profile
    media_profile = await media.GetProfiles()[0]

    # Get PTZ configuration options for getting continuous move range
    request = ptz.create_type('GetConfigurationOptions')
    request.ConfigurationToken = media_profile.PTZConfiguration.token
    ptz_configuration_options = await ptz.GetConfigurationOptions(request)

    global moverequest
    moverequest = ptz.create_type('ContinuousMove')
    moverequest.ProfileToken = media_profile.token
    if moverequest.Velocity is None:
        moverequest.Velocity = await ptz.GetStatus(
            {
                'ProfileToken': media_profile.token,
            },
        ).Position

    # Get range of pan and tilt
    # NOTE: X and Y are velocity vector
    global XMAX, XMIN, YMAX, YMIN
    XMAX = ptz_configuration_options.Spaces.ContinuousPanTiltVelocitySpace[0].XRange.Max
    XMIN = ptz_configuration_options.Spaces.ContinuousPanTiltVelocitySpace[0].XRange.Min
    YMAX = ptz_configuration_options.Spaces.ContinuousPanTiltVelocitySpace[0].YRange.Max
    YMIN = ptz_configuration_options.Spaces.ContinuousPanTiltVelocitySpace[0].YRange.Min


def readin():
    """Reading from stdin and displaying menu."""
    global moverequest, ptz

    selection = sys.stdin.readline().strip('\n')
    lov = [x for x in selection.split(' ') if x != '']
    if lov:
        loop = asyncio.get_event_loop()
        if lov[0].lower() in ['u', 'up']:
            coro = move_up(ptz, moverequest)
        elif lov[0].lower() in ['d', 'do', 'dow', 'down']:
            coro = move_down(ptz, moverequest)
        elif lov[0].lower() in ['l', 'le', 'lef', 'left']:
            coro = move_left(ptz, moverequest)
        elif lov[0].lower() in ['l', 'le', 'lef', 'left']:
            coro = move_left(ptz, moverequest)
        elif lov[0].lower() in ['r', 'ri', 'rig', 'righ', 'right']:
            coro = move_right(ptz, moverequest)
        elif lov[0].lower() in ['ul']:
            coro = move_upleft(ptz, moverequest)
        elif lov[0].lower() in ['ur']:
            coro = move_upright(ptz, moverequest)
        elif lov[0].lower() in ['dl']:
            coro = move_downleft(ptz, moverequest)
        elif lov[0].lower() in ['dr']:
            coro = move_downright(ptz, moverequest)
        elif lov[0].lower() in ['s', 'st', 'sto', 'stop']:
            coro = ptz.Stop({'ProfileToken': moverequest.ProfileToken})
        else:
            print(
                "What are you asking?\tI only know, 'up','down','left','right', "
                "'ul' (up left), \n\t\t\t'ur' (up right), 'dl' (down left), "
                "'dr' (down right) and 'stop'",
            )

    if coro:
        loop.call_soon(coro)
    print('')
    print('Your command: ', end='', flush=True)


if __name__ == '__main__':
    setup_move()
    loop = asyncio.get_event_loop()
    try:
        loop.add_reader(sys.stdin, readin)
        print('Use Ctrl-C to quit')
        print('Your command: ', end='', flush=True)
        loop.run_forever()
    except Exception:
        pass
    finally:
        loop.remove_reader(sys.stdin)
        loop.close()



import asyncio
import datetime as dt
import logging

import pytz

import onvif


logging.getLogger('zeep').setLevel(logging.DEBUG)


async def run():
    mycam = onvif.ONVIFCamera(
        '192.168.3.10',
        80,
        'hass',
        'peek4boo',
        wsdl_dir='/home/jane/onvif/wsdl',
    )
    await mycam.update_xaddrs()

    if not await mycam.create_pullpoint_subscription():
        print('PullPoint not supported')
        return

    event_service = mycam.get_service('events')
    properties = await event_service.GetEventProperties()
    print(properties)
    capabilities = await event_service.GetServiceCapabilities()
    print(capabilities)

    pullpoint = mycam.create_pullpoint_service()
    await pullpoint.SetSynchronizationPoint()
    req = pullpoint.create_type('PullMessages')
    req.MessageLimit = 100
    req.Timeout = dt.timedelta(seconds=30)
    messages = await pullpoint.PullMessages(req)
    print(messages)

    subscription = mycam.create_subscription_service('PullPointSubscription')
    req = subscription.zeep_client.get_element('ns5:Renew')
    req.TerminationTime = str(dt.datetime.now(pytz.UTC) + dt.timedelta(minutes=10))
    termination_time = (dt.datetime.now(pytz.UTC) + dt.timedelta(minutes=10)).isoformat()
    await subscription.Renew(termination_time)
    await subscription.Unsubscribe()


if __name__ == '__main__':
    loop = asyncio.get_event_loop()
    loop.run_until_complete(run())



import setuptools


def _get_long_description():
    with open('README.rst') as readme_file:
        return readme_file.read()


setuptools.setup(
    name='aonvif',
    use_scm_version=True,
    description='ONVIF asynchronous client implementation in Python',
    long_description=_get_long_description(),
    author='Andrey Martyanov',
    author_email='andrey@martyanov.com',
    url='https://github.com/martyanov/aonvif',
    license='MIT',
    keywords=['onvif', 'client', 'asyncio'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    zip_safe=False,
    project_urls={
        'Bug Reports': 'https://github.com/martyanov/onvif/issues',
        'Repository': 'https://github.com/martyanov/onvif',
    },
    packages=setuptools.find_packages(
        exclude=[
            'examples',
            'tests',
        ],
    ),
    package_data={
        '': [
            '*.rst',
        ],
        'onvif.wsdl': [
            '*.wsdl',
            '*.xsd',
            '*xml*',
            'envelope',
            'include',
            'addressing',
        ],
    },
    python_requires='>=3.7,<4',
    setup_requires=[
        'setuptools_scm==6.3.2',
    ],
    install_requires=[
        'zeep[async]>=4,<5',
    ],
    extras_require={
        'dev': [
            'flake8-broken-line==0.4.0',
            'flake8-bugbear==21.11.29',
            'flake8-commas==2.1.0',
            'flake8-comprehensions==3.7.0',
            'flake8-isort==4.1.1',
            'flake8-quotes==3.3.1',
            'flake8==4.0.1',
            'pep8-naming==0.12.1',
            'twine==3.7.1',
        ],
        'test': [
            'pytest-asyncio==0.16.0',
            'pytest-cov==3.0.0',
            'pytest==6.2.5',
        ],
    },
    entry_points={
        'console_scripts': [
            'onvif-cli=onvif.cli:main',
        ],
    },
)



import asyncio

import onvif


async def rotate_image_180():
    """Rotate the image."""

    # Create the media service
    mycam = onvif.ONVIFCamera('192.168.0.112', 80, 'admin', '12345')
    await mycam.update_xaddrs()
    media_service = mycam.create_media_service()

    profiles = await media_service.GetProfiles()

    # Use the first profile and Profiles have at least one
    _ = profiles[0].token

    # Get all video source configurations
    configurations_list = await media_service.GetVideoSourceConfigurations()

    # Use the first profile and Profiles have at least one
    video_source_configuration = configurations_list[0]

    # Enable rotate
    video_source_configuration.Extension[0].Rotate[0].Mode[0] = 'OFF'

    # Create request type instance
    request = media_service.create_type('SetVideoSourceConfiguration')
    request.Configuration = video_source_configuration

    # ForcePersistence is obsolete and should always be assumed to be True
    request.ForcePersistence = True

    # Set the video source configuration
    await media_service.SetVideoSourceConfiguration(request)


if __name__ == '__main__':
    loop = asyncio.get_event_loop()
    loop.run_until_complete(rotate_image_180())
